import os
import re
import sys
import time
from termcolor import cprint, colored

j = os.path.join

def to_save(line,ty,root_dir):
    if not os.path.exists(os.path.join(root_dir,ty)):
        with open(os.path.join(root_dir,ty), "w") as fp:
            pass

    with open(os.path.join(root_dir,ty), "a+") as fp:
        print(line, file=fp)

def file_search(info, fs):
    for f in fs:
        cprint("--> file: %15s" % colored(f,attrs=['bold']), 'yellow', file=sys.stderr)
        with open(f) as fp:
            dic = {}
            for line in fp:
                l = line.strip()
                if re.search(re.compile('(%s)' % info), l):
                    yield l

def call_vim(tmp_file="/tmp/add.log.tmp.log"):
    from fabric.api import output
    from fabric.api import local
    output.running = False
    local("vim %s" % tmp_file)
    time.sleep(0.5)
    with open(tmp_file) as fp:
        text = fp.read()
        return text


#!/usr/bin/env python3
# encoding: utf-8
import os
import sys
import atexit
import signal
import argparse


def cmd():
    doc = """
            this is system for analyze data from weather 
            written by qingluan
    """
    parser = argparse.ArgumentParser(usage=" how to use this ", description=doc)
    parser.add_argument("-d", "--daemon", default=None, help="daemon start operator\n[moniter, rpc, test]")
    parser.add_argument("--start", default=False, action="store_true")
    parser.add_argument("--stop", default=False, action="store_true")
    parser.add_argument("-a", "--auth-token", default="hello", help="rpc server authentication's token")
    parser.add_argument("-p", "--rpc-port", default=17000, help="rpc server port ")
    parser.add_argument("-H", "--host", default="localhost", help="rpc server bind ip")
    parser.add_argument("-c", "--rpc-client", default=False, action='store_true',help="rpc client to connect to remote rpc , this is interact mode")
    return parser.parse_args()


def daemonize(pidfile, *, stdin='/dev/null',
              stdout='/dev/null',
              stderr='/dev/null'):
    if os.path.exists(pidfile):
            raise RuntimeError('Already running')
    # First fork (detaches from parent)
    try:
        if os.fork() > 0:
            raise SystemExit(0)
    except OSError as e:
        raise RuntimeError('fork #1 failed.')
    os.chdir('/')
    os.umask(0)
    os.setsid()
    # Second fork (relinquish,session leadership)
    try:
        if os.fork() > 0:
            raise SystemExit(0)
    except OSError as e:
        raise RuntimeError('fork #2 failed.')
    # Flush I/O buffers
    sys.stdout.flush()
    sys.stderr.flush()
    # Replace file descriptors for stdin, stdout, and stderr
    with open(stdin, 'rb', 0) as f:
        os.dup2(f.fileno(), sys.stdin.fileno())
    with open(stdout, 'ab', 0) as f:
        os.dup2(f.fileno(), sys.stdout.fileno())
    with open(stderr, 'ab', 0) as f:
        os.dup2(f.fileno(), sys.stderr.fileno())
        # Write the PID file
    with open(pidfile, 'w') as f:
        print(os.getpid(), file=f)
    # Arrange to have the PID file removed on exit/signal
    atexit.register(lambda: os.remove(pidfile))
    # Signal handler for termination(required)

    def sigterm_handler(signo, frame):
        raise SystemExit(1)
    signal.signal(signal.SIGTERM, sigterm_handler)


def test(): 
    import time
    sys.stdout.write('Daemon started with pid {}\n'.format(os.getpid()))
    while True:
        sys.stdout.write('Daemon Alive! {}\n'.format(time.ctime()))
        time.sleep(10)


def run(func):
    PID_F = os.path.join("/var/run/", func.__name__ + ".pid")
    LOG_F = os.path.join("/var/log/", func.__name__ + ".normal.log")
    ERROR_F = os.path.join("/var/log/", func.__name__ + ".error.log")
    try:
        print(func.__name__)
        daemonize(PID_F, stdout=LOG_F, stderr=ERROR_F)
    except RuntimeError as e:
        print(e, file=sys.stderr)
        raise SystemExit(1)
    func()


def stop(func):
    PID_F = os.path.join("/var/run/", func.__name__ + ".pid")
    LOG_F = os.path.join("/var/log/", func.__name__ + ".normal.log")
    for i in [PID_F]:
        if os.path.exists(i):
            with open(i) as f:
                os.kill(int(f.read()), signal.SIGTERM)
        else:
            print("Not running : ", i, file=sys.stderr)
    raise SystemExit(1)
    from time import asctime
    print(asctime() ," -- stop --", file=LOG_F)


def main():
    args = cmd()
    func = None
    if args.daemon == "monitor":
        func = monitor

    if args.daemon == "rpc":
        from rpc.run import RPC_server_start
        from functools import partial
        func = partial(RPC_server_start, args.host, args.rpc_port)
        func.__name__ = "rpc"

    if args.daemon == "test":
        func = test

    if args.start and func:
        run(func)
    elif args.stop and func:
        stop(func)
    else:
        print("you must choose a operator : start/stop", file=sys.stderr)

    if args.rpc_client:
        from rpc import get_client, interact
        interact(get_client((args.host, args.rpc_port), args.auth_token.encode()))

if __name__ == "__main__":
    main()



import os, sys
import random
from urllib.parse import urljoin
BASE_URL = os.getenv("Q_PATH")
sys.path.append(BASE_URL)


from qlib.asyn import Exe
from qlib.net import to
_exe = Exe(6)

_GEO_IP = [
    # 0: "http://ip-api.com/json/",
    "https://freegeoip.net/json/",
]

def check_local(ip):
    if ip == "127.0.0.1":
        return "hangzhou "
    _LOCAL_IP = {
        "10":True,
        "172" : lambda x: int(x.split(".")[1]) > 15,
        "192" : lambda x: int(x.split(".")[1]) == 168,
    }

    head = ip.split(".")

geo_api = _GEO_IP[random.randint(0, len(_GEO_IP)-1)]



def search_ip(*ips, callback=None, asyn=False):
    e = [urljoin(geo_api,ip) for ip in ips]
    
    def get_json(url):
        return url, to(url).json()
    if asyn:
        for url in e:
            _exe.done(get_json, callback, url)
        return True

    res = _exe.map(get_json, e)
    if callback:
        callback(res)    
    else:
        return res
    #     return parse()
    # else:
        
    #     @_exe(callback=callback)
    #     def parse():
    #         @network(e, timeout=8)
    #         def _e():
    #             return _e.res.json()
    #         return _e()
    # return parse()



import os
import re
import ctypes
from ctypes import *
from functools import wraps

from termcolor import cprint

j = os.path.join

qlib_c_lib_path = j(os.getenv("HOME"), ".Q/clib/")
uchar_p_size = 1024


P_C_BASETYPE = {
    int: c_int,
    str: c_char_p,
    float: c_float,
}


P_C = {
    str: lambda x:c_char_p(x.encode("utf8", "ignore")),
    int: lambda x:c_long(x) if x> 65535 else c_int(x),
    float: lambda x:c_float(x),
}

extend_table = {
    'c_uchar': c_ubyte,
    'c_wchar_t': c_wchar,
    'c_uu_char': c_ubyte,
    'c_u_char': c_ubyte,
    'c_u_char_p': c_ubyte * uchar_p_size,
    'c_wchar_t_p': c_wchar_p,
    'c_uchar_p': c_ubyte * uchar_p_size,
    'c_void': c_voidp,
}


def filter_decor(c):
    """
    this function for to remove parameter decorator .
        like this 'const char some[100]' 
    """
    return re.sub(r'(const|extern)', '', c)



def get_base_ctype(c_type):
    words = c_type.strip().split()
    c_type = c_type.strip()
    l = len(words)
    c_type_name = None
    
    if l == 1:
        c_type_name = 'c_' + c_type
    else:
        
        # deal with 'char*' 
        if words[-1] == '*' and l==2:
            pre = re.sub(r' ', '', c_type)
            c_type_name = 'c_' + pre
        else:
            # let 'unsigned char *' to 'uchar*'
            pre = re.sub(r'(^\w+?) |( )', '', c_type)
            c_type_name = 'c_' + words[0][0] + pre
    
        # let 'c_uchar*' to 'c_uchar_p'
        c_type_name = c_type_name.replace(r'*', '_p')


    if hasattr(ctypes, c_type_name):
        return getattr(ctypes, c_type_name)
    else:
        try:
            return extend_table[c_type_name]
        except KeyError:
            return c_type


def check_pointer(n):
    """
    this function for seprate name and point 
    like this  'char *' -> ['char', 1] ,
    'char **' -> ['char', 2] 

    if it is not pointer , will return None
    like this 'char' -> ['char', 0]
    """
    w = n.split("*")
    return w[0].strip(), len(w) -1 



def to_c(py):
    try:
        return None, P_C[type(py)](py)
    except KeyError:
        return None, py


def array_to_c(lst):
    if isinstance(lst, list) and not isinstance(lst[0], list):
        struct = P_C_BASETYPE[type(lst[0])] * len(lst)
        vals = [to_c(i) for i in lst]
        return struct, struct(*vals)
    else:
        l = len(lst)
        arr = [ array_to_c(i) for i in lst]
        sub_vals = [i[1] for i in arr]
        sub_struct = arr[0][0]
        struct = sub_struct * l
        val = struct(*sub_vals)
        return struct, val 


def deref(addr, typ):
    if typ:
        return ctypes.cast(addr, ctypes.POINTER(typ)).contents
    return None


def c(func):
    """
    translaste normal py's arguments to c's arguments
    """
    # def _c(func):

    @wraps(func)
    def _wrap(*args):
        instance = args[0]
        c_args = [instance]  # keep self
        c_args += [ii[1] for ii in [ array_to_c(i) if isinstance(i, list) else to_c(i) for i in args[1:]]]
        return func(*c_args)

    return _wrap

    # return _c

def c_restype(cfun, restype):

    def _c(func):

        @wraps(func)
        def _d(*args, **kargs):
            instance = args[0]
            if hasattr(instance, cfun):
                f = getattr(instance, cfun)
                if hasattr(f, 'restype'):
                    f.restype = ctypes.POINTER(restype)

            return func(*args, **kargs)

        return _d
    return _c



def include(path, structs):

    def __include(cls):
        """
        to auto load  c dll lib
        """
        def _include(name):
            f = j(path, name + ".h")
            txt = None
            with open(f) as fp:
                txt = fp.read()
            funlist = re.findall(r'(?:\s)([\*\w ]+?\s\w+?\([\w,\s\*]{0,100}\)+.)' ,txt)
            for i in funlist:
                yield re.findall(r'(.+?\s)(\w+?)\(', i)[0]

        def set_restype(retur):
            r = get_base_ctype(filter_decor(retur.strip()))
            if isinstance(r, str):
                struct,num = check_pointer(r)
                if hasattr(structs, struct):
                    e = getattr(structs, struct)
                    cprint("load:" + struct, "green")
                else:
                    cprint(r + " ignored", "yellow")
                    return None

                if num > 0:
                    return ctypes.POINTER(e)
                return e
            else:
                return r


        def _wrap():

            lib = CDLL(j(path, cls.__name__ + ".so"))
            for retu, func_s in _include(cls.__name__):
                func = getattr(lib, func_s)
                rest = set_restype(retu)
                if rest:
                    func.restype = rest
                setattr(cls, func_s, func)
        _wrap()
        return cls
    return __include


def gen_py_ctype(line, uchar_p_size=1024):
    """
    read lib's header file
    can parse c's parameter , but notion!!!!!
    array must be this 'xxx[12]'  not 'xxx [12]', !!!! no space between them.!!!
    """

    def escape(line):
        """
        this function for to let translate multi-space to single-space.
        like this 'const     char   * ss' -> 'const char * ss'
        """
        return re.sub(r'([ ]{2,})', ' ', line)


    def check_array(n):
        """
        this function for to find a array parameter.
            like this 'char some[100]' 
        """
        d = re.findall("(?:\[)(\d+?)\]", n)
        if d:
            return n.split("[")[0], int(d[0])
        else:
            return n, 1

    words = escape(line.strip()).split()
    name = words.pop()
    c_type = filter_decor(" ".join(words))
    name, arr_num = check_array(name)

    c = get_base_ctype(c_type)

    ## check c , ensure it is not null

    if arr_num > 1:
        c = c * arr_num

    return (name, c)


class C(Structure):
    """
        a interface need to immplementation
    """
    def __getitem__(self,k ):
        if hasattr(self, k):
            return getattr(self, k)
        return None

    def keys(self):
        return [i for i in self.__dir__() if not i.startswith("__")]

    def to_array(self, arr, len):
        return arr[:len]


def structure(u_char_size):
    """
        a auto generate a py structure.
        but !!! this can only use for simple struct ,
        !! not supported 'nested' struct
    """
    _uchar_p_size = u_char_size

    def _struct(cls):
        if hasattr(cls, '_raw'):
            txt = cls._raw
        else:
            raise Exception("Must set a _raw ")
        head, body_tail = txt.split("{")
        body, tail = body_tail.split("}")
        # cls._struct_name = re.findall(r'(?:struct[ ]{1,10})(\w+?)\{', txt)[0]
        all_ele = re.findall(r'(?:\s)([\w \*\[\d\]]{0,20})\;', body)
        
        _fields_ = []
        for l in all_ele:
            _fields_.append(gen_py_ctype(l.strip(), uchar_p_size=_uchar_p_size))
        
        return type(cls.__name__, (C,), {'_fields_': _fields_})
    return _struct




from .__file import to_save, call_vim, file_search

__all__ = [
    'to_save',
    'call_vim',
    'file_search',
    'j',
]


def j(*paths):
    import os
    from functools import reduce
    return reduce(os.path.join, paths)


import os, sys, time
sys.path.append(os.getenv("Q_PATH"))

from termcolor import cprint, colored
from contextlib import contextmanager

from qlib.io.console import stdout
from qlib.asyn import Exe


INFO = 0x08
ERR = 0x00
OK = 0x04
WRN = 0x02
FAIL = 0x01



def tag_print(tag, *args, tag_color='red', tag_attr=['bold', 'blink'], txt_color='grey', txt_attr=[], **kargs):
    tag = "[%s] " % colored(tag, tag_color, attrs=tag_attr)
    txt = colored(' '.join([str(i) for i in args]), txt_color, attrs=txt_attr)
    print(tag + txt, **kargs)



class LogControl:
    INFO = 0x08
    ERR = 0x00
    OK = 0x04
    WRN = 0x02
    FAIL = 0x01
    LOG_LEVEL = None

    @staticmethod
    def err(*args, txt_color='grey', txt_attr=['bold'], **kargs):
        tag_print('err', *args, txt_color=txt_color, txt_attr=txt_attr, **kargs)

    @staticmethod
    def info(*args, txt_color='grey', txt_attr=[], **kargs):
        if LogControl.LOG_LEVEL & INFO:
            tag_print('info', *args,  tag_color='cyan', tag_attr=['bold'],  txt_color=txt_color, txt_attr=txt_attr, **kargs)

    @staticmethod
    def wrn(*args, txt_color='grey', txt_attr=[], **kargs):
        if LogControl.LOG_LEVEL & WRN:
            tag_print('warning', *args,  tag_color='yellow', tag_attr=['bold'],  txt_color=txt_color, txt_attr=txt_attr, **kargs)

    @staticmethod
    def ok(*args, txt_color='grey', txt_attr=[], **kargs):
        if LogControl.LOG_LEVEL & OK:
            tag_print('√', *args,  tag_color='green', tag_attr=['bold'],  txt_color=txt_color, txt_attr=txt_attr, **kargs)

    @staticmethod
    def fail(*args, txt_color='grey', txt_attr=[], **kargs):
        if LogControl.LOG_LEVEL & FAIL:
            tag_print('X', *args,  tag_color='red', tag_attr=['bold'],  txt_color=txt_color, txt_attr=txt_attr, **kargs)


@contextmanager
def running(msg, interval=1):
    running = True
    # tag = ['| ', '/ ', '- ', '\\ ']
    def _timer():
        m = 0
        while running:
            print(' ...', end='')
            sys.stdout.flush()
            time.sleep(interval)

    e = Exe(2)
    try:
        info(msg, end=' ')
        e.submit(_timer)
        with stdout(None) :
            yield

    except Exception as e:
        running = False
        err(e)
        sys.exit(0)
    else:
        running = False
        print("\b\b\b" , end='')
        cprint('[%s]' % colored('√', 'green', attrs=['bold']))
    finally:
        running = False



import requests as R
import urllib
from zoomeye import Zoomeye, Base

class Shodan(Zoomeye):
    
    def __init__(self):
        Base.main_url = "https://api.shodan.io"
        Base.urls = {
            "host":"GET /shodan/host/search",
        }
        Base.token =  "XHSWncMjN6MEyekECTMcOeoEocl6VO2q"
    
        self.session = R.Session()
        self.headers = self.session.headers

    def _u(self, url):
        url = super(Shodan, self)._u(url)
        token_url = urllib.parse.urljoin(url, "?key=%s&" % Shodan.token)
        return token_url


# -*- coding: utf8 -*- 
import os, json, sys

from qlib.net import to, parameters
from qlib.net.agents import AGS
from qlib.graphy import random_choice
from qlib.asyn import Exe
from qlib.log import LogControl
from qlib.data.sql import SqlEngine

from lxml.etree import HTML
from urllib.parse import quote

LogControl.LOG_LEVEL = LogControl.OK
LogControl.LOG_LEVEL |= LogControl.FAIL
LogControl.LOG_LEVEL |= LogControl.INFO

class Searcher:
    """
    @proxy is setting request's proxy 
        sample: {
            'http': 'socks5://127.0.0.1:1080',
            'https': 'socks5://127.0.0.1:1080'
        }
    """
    config = {
        'search_url': '',
        'search_args': {},
    }

    def __init__(self, ssl=True, asyn=True, debug=False, db=False, database=None, proxy=False):
        self.url_pre = 'https://www.' if ssl else  'https//www.'
        self.search_name = self.__class__.__name__.lower()
        self.host = self.url_pre + self.search_name + '.com'
        self.agent = random_choice(AGS)
        self.asyn = None
        self.db = None
        self.debug =debug
        self.proxy = None
        self.grep_html_content = dict()

        if asyn:
            self.asyn = Exe(20)

        if proxy:
            LogControl.info("loading proxy..", end='')
            self.proxy = proxy #Mongo('local').find("setting")[0].get("proxy")
            if self.proxy:
                LogControl.ok("")


        if db:
            self.use_db = db
            db_path = os.path.join(os.getenv("HOME"), '.Search-engine-sqlite3-db.db') if not database else database
            self.db_path = db_path
            self.db = SqlEngine(database=db_path)
            if not self.db.table_list():
                self.db.create(self.search_name, query=str, content=str, type='web')

    def proxy_to(self, url, **kargs):
        if self.proxy:
            return to(url, proxy=self.proxy, **kargs)
        else:
            return to(url, **kargs)

    def xpath(self, html, *tags,exclude=None):
        xhtml = HTML(html)
        exclude = '[not(name()={})]'.format(exclude) if exclude else ''
        LogControl.info("//" + "//".join(tags) + exclude) if self.debug else ''
        for item in xhtml.xpath("//" + "//".join(tags) + exclude):
            yield item

    def ready(self, pre_url, *args, **options):
        pass

    def set_args(self, **kargs):
        return parameters(**kargs)

    def grep(self, url, grep_str, num):
        """
        async to grep url's content by grep_str, then will return (num, [*url]) -> callback

        """

        def _request():
            if  grep_str.lower().find(url.split(".").pop().lower()) != -1:
                return num, []
            res = self.proxy_to(url)
            if res.status_code / 100 ==2:

                encoding = 'utf8'#res.encoding if res.encoding else 'utf8'
                LogControl.err(url, res.encoding)
                if res.content:
                    try:
                        h_content = res.content.decode(encoding)
                        return num, [ i.attrib.get('href') for i in HTML(h_content).xpath('//a') if i.attrib.get('href', '').find(grep_str) != -1]
                    except UnicodeError as e:
                        LogControl.err(e,'\n', res.encoding, encoding, url)
                        return num, []
                    except KeyError as e:
                        LogControl.err(e, url)
                        return num, []

                    
            else:
                return num, []
        if self.asyn:
            self.asyn.done(_request, self.save_grep)

    def save_grep(self, num, res):
        self.grep_html_content[num] = res

class Google(Searcher):
    pass


class Baidu(Searcher):
    
    config = {
        'translate_tag': "@class='op_dict_content'",
        'translate_url': "https://www.baidu.com/s?ie=utf-8&f=8&rsv_bp=0&rsv_idx=1&tn=baidu&wd={query}\
                            &rsv_pq=be1d9cdc00001f3a&rsv_t=e851oZevI5oKNV9F6W9CwY5wqlPyiK9Pqz0RLr7vdyawQru3NgKGrAcsizA\
                            &rqlang=cn&rsv_enter=1&rsv_sug3=20&rsv_sug2=0&inputT=2703&rsv_sug4=6023"
    }
    

    def translate(self, raw, to_lang='英文'):
        if not self.asyn:
            self.asyn = Exe(10)

        def query(url):
            res = to(url)
            en = res.encoding
            return res.status_code, res.content.decode(en) 

        def display(code, content):
            '''
            display content
            '''
            if int(code / 200) == 1:
                res = '\n'.join([ i.text for i in self.xpath(content, "div[%s]" % Baidu.config['translate_tag'], 'td', '*')])
                LogControl.ok(res)
            else:
                LogControl.fail("not found this")

        data = Baidu.config['translate_url'].format(query=quote(raw + ' ' + to_lang))
        self.asyn.done(query, display, data)



class BaiduTranslate(Searcher):

    def translate(self, raw, from_lan='en', to_lan='zh', detail=False, example=False):
        """
        
        detail : if detail == true will get more information.
        example : if example == true will get some sample.

        """
        self.host = 'fanyi.baidu.com/v2transapi'
        if not self.asyn:
            self.asyn = Exe(10)

        def query(data):
            res = to(self.host, method='post', data=data)
            return res.status_code, res.json()

        def tree_dict(d, func, key=None, lfunc=None):
            if isinstance(d, dict):
                for item in d:
                    tree_dict(d[item], func, key=item)
            elif isinstance(d, list):
                for item in d:
                    if lfunc:
                        tree_dict(item, lfunc)
                    else:
                        tree_dict(item, func)
            else:
                if key:
                    if key == 'dst':
                        LogControl.ok(d, txt_color='green', txt_attr=['underline','bold'])
                    else:
                        func(d, key)
                else:
                    func(d)

        def single_display(val, key=None, level=1):
            if key:
                LogControl.ok('[' + key + ']', '\t', val)
            else:
                LogControl.info('\t\t' + '   ' * level, val)

        def display(code, content):
            '''
            display content
            '''
            if int(code / 200) == 1:
                try:
                    tree_dict(content['trans_result'], single_display, 'base')
                    if detail:
                        tree_dict(content['dict_result'], single_display, 'dict')
                    if example:
                        tree_dict(content['liju_result'], single_display, 'sample')
                except KeyError:
                    LogControl.fail("Only: ", *content.keys())
                    for k in content:
                        LogControl.err(k, content[k])

            else:
                LogControl.fail("not found this")

        data = {
            'from': from_lan,
            'to': to_lan,
            'query': raw,
            'transtype':'translang', 
            'simple_means_flag':'3',
        }
        self.asyn.done(query, display, data)


class DuckDuckGo(Searcher):

    """
    @db=False/True : will use sqlite3 database .
    @database= 'path' # specify a path to save sqlite database file. default is ~/.Search-engine-sqlite3-db.db

    supported web , news, video, images
    """

    def __init__(self, *args, **kargs):
        super().__init__(*args, **kargs)
        self.search_url = 'https://duckduckgo.com/{use_js}?q={query}&{options}&vqd={vqd}'
        self.search_pools = {
            'web': 'd.js',
            'news': 'news.js',
            'video': 'v.js',
            'images': 'i.js',
        }

        self.search_args = 'l=wt-wt&p=1&s=0&a=h_&ct=US&ss_mkt=us&o=json' # output: json
        self.result = None
        self.last_search_type = 'web'
        self.next_url = None


    # def search_old(self, query, type='web'):
    #     """
    #     Abandoned function 
    #     """
    #     url_query = quote('+'.join(query.split()))

    #     def get_vqd(query):
    #         sss = to('http://duckduckgo.com/?q={query}&t=h_&ia={type}'.format(query=query, type=type)).content.decode('utf8')
    #         return sss[sss.rfind("vqd"):].split("&").pop(0).split("=").pop()

    #     args = self.search_args
    #     vqd = get_vqd(url_query)
    #     url = self.search_url.format(query=url_query, options=self.search_args, vqd=vqd) + '&sp=1&yhs=1'
    #     LogControl.info(url) if self.debug  else ''
    #     response = to(url, headers={'cookie':'ak=-1'})
    #     return response.status_code, response 

    def search(self, query, type='web', url=None):
        """
        supported web , news, video, images
        """

        def get_vqd(query):
            vqd_url = 'https://duckduckgo.com/?q={query}&t=h_&ia={type}'.format(query=query, type=type)
            LogControl.info(vqd_url) if self.debug else ''
            sss = self.proxy_to(vqd_url).content.decode('utf8')
            return sss[sss.rfind("vqd"):].split("&").pop(0).split("=").pop()


        if url is None:
            url_query = '+'.join(query.split())


            args = self.search_args
            vqd = get_vqd(url_query)
            url = self.search_url.format(use_js=self.search_pools[type], query=quote(query), options=args, vqd=vqd) + '&sp=1&yhs=1'

            

        LogControl.info(url) if self.debug  else ''
        response = self.proxy_to(url, headers={'cookie':'ak=-1'})
        if response.status_code / 100 == 2:
            
            self.last_search_type = type # record successful request's type
            json_content = ''
            try:
                json_content = response.json()
            except Exception:
                LogControl.err(response.content)
                sys.exit(0)

            self.result = json_content.get('results')
            self.deepanswer = json_content.get('deep_answers')
            self.next_url = self.result[-1].get('n')
            
            if self.use_db:
                self.db.insert(self.search_name, ['query', 'content', 'type'], query, json.dumps(json_content), type)

            return json_content.get('results')
        else:
            LogControl.err(response.status_code, 'err')
            return ''

    def next(self):
        """
        see next page 's result simplely.
        """

        if not self.next_url:
            LogControl.fail("no next url found !")
            return ''
        return self.proxy_to(self.host + self.next_url).json()

    def more(self):
        """
        like next
        """
        return self.next()

    def search_images(self, query):
        return self.search(query, type='iamges')

    def search_video(self, query):
        return self.search(query, type='video')

    def search_news(self, query):
        return self.search(query, type='news')



import re


def extract(string, pre='(', back=')'):
    """
    Can extract content between from spic[0] and spic[1], default spic is '()' , which from string.
    """

    l_count = 0
    while string.find(pre) != -1:
        l_count += 1
        s = string.find(pre) + len(pre)
        string = string[s:]

        last_b = string.rfind(back)
        next_s = string.find(pre)
        next_b = string.find(back)
        if next_b < next_s:
            yield string[:next_b]
            string = string[next_b:] 
        else:
            yield string[:last_b]
            string = string[:last_b]
        if string is None:
            break


def get_from_func(string, name):

    while string.find(name) != -1:
        string = string[string.find(name) + len(name):]
        yield next(extract(string))
        if not string:
            break


def get_from_tag(string, tag):
    if isinstance(string, bytes):
        string = string.decode("utf8")
    g_r = re.compile(r'(?:\<{tag}.*?\>)(.+?)(?:</{tag}>)'.format(tag=tag))
    if string is None:
        return

    for item in re.findall(g_r, string):
        yield item
    # pre='<%s>' % tag
    # back = '</%s>' % tag
    # if string is None:
    #     return ''
    # while string.find(tag) != -1:
    #     yield next(extract(string, pre=pre, back=back))
    #     string = string[string.find(pre) + len(pre):]
    #     if string is None:
    #         break


def replace_tag_to(string, func=False, **tags):
    """
    relace tag's content to value in tags

    @func = False/True
        while set True , will use tags's value should pass callback function, not value

    """
    

    def _tag(tag):
        return '<%s>' % tag, '<%s/>' % tag

    for tag in tags:
        val = tags[tag]
        replace_r = r'\<{tag}.*?\>.+?\<\/{tag}\>'.format(tag=tag)
        for raw in get_from_tag(string, tag):
            if not func:
                # string = string.replace('<{tag}>{raw}</{tag}>'.format(tag=tag, raw=raw), val)
                string = re.sub(replace_r, val, string)
            else:
                string = re.sub(replace_r, val(raw), string)
                # string = string.replace('<{tag}>{raw}</{tag}>'.format(tag=tag, raw=raw), val(raw))

    return string


from .config import RAW_HEADERS as header
from .__requester import set_setssion, session, network, to, parameters
from .agents import AGS

__all__ = [
    'header',
    'network',
    'set_setssion',
    'session',
    'to',
    'parameters',
    'AGS',
]


import requests
from functools import wraps
from .config import RAW_HEADERS
from urllib.parse import urlencode
# _methods = {}


def parameters(**pars):
    return urlencode(pars)


def set_setssion():
    return requests.session()


def session(url):

    def _wrap(func):
        session = requests.Session()

        @wraps(func)
        def run(*args, **kargs):
            session.get(url)
            kargs['session'] = session
            return func(*args, **kargs)
        return run
    return _wrap


def to(url, data=None, ssl=False, method='get', proxy=None, **option):
    if not url.startswith("http"):
        if ssl:
            url = 'https://' + url
        else:
            url = 'http://' + url

    headers = RAW_HEADERS
    if 'headers' in option:
        for k in option['headers']:
            headers[k] = option['headers'][k]

    if "seesion" in option:
        m = option['session']
    else:
        m = getattr(requests, method)
        if proxy:
            return m(url, data=data, headers=headers, proxies=proxy)
        else:
            return m(url, data=data, headers=headers)

def network(url, data=None, method="get", **options):
    if "session" in options:
        m = getattr(options['session'], method)
    else:
        m = getattr(requests, method)

    if not data:
        req = m(url, headers=RAW_HEADERS)
    req = m(url, data=data, headers=RAW_HEADERS)

    def _wrap(func):
        func.res = req

        @wraps(func)
        def __call_back(*args, **kargs):
            return func(*args, **kargs)
        return __call_back
    return _wrap



RAW_HEADERS  = {
    'Accept-Encoding': 'gzip, deflate, sdch',
    'Accept-Language': 'en-US,en;q=0.8',
    'Cache-Control': 'max-age=0',
    'Connection': 'keep-alive',
    # 'Host': None,
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.106 Safari/537.36'
}